///////////////////////////////////////////////
//// XTR.UnicodeString.cpp
//// (C) 2006 James Bernard Schumacher III
///////////////////////////////////////////////

#include <windows.h>
#include "XTR.String.hpp"

namespace XTR
{

UnicodeString::UnicodeString() throw() : m_pString(NULL),m_dwBufferSize(0)
{
    m_pString = new wchar_t[16];
    if (m_pString != NULL)
    {
        m_dwBufferSize = 16;
        m_pString[0] = 0;
    }
}

UnicodeString::UnicodeString(unsigned long dwBufferSize) throw() : m_pString(NULL),
     m_dwBufferSize(0)
{
    m_pString = new wchar_t[dwBufferSize];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwBufferSize;
        m_pString[0] = 0; 
    }
}

UnicodeString::UnicodeString(wchar_t ch) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    m_pString = new wchar_t[16];
    if (m_pString != NULL)
    {
        m_dwBufferSize = 16;
        m_pString[0] = ch;
        m_pString[1] = 0;
    }
}

UnicodeString::UnicodeString(const wchar_t * pString) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    const unsigned long dwLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwLength);
    m_pString = new wchar_t[dwAllocate];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwAllocate;
        StringCopy(m_pString,pString,dwLength);
        m_pString[dwLength] = 0;
    }
}

UnicodeString::UnicodeString(const UnicodeString & str) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    const unsigned long dwLength = str.GetLength();
    const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwLength);
    m_pString = new wchar_t[dwAllocate];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwAllocate;
        StringCopy(m_pString,str.m_pString,dwLength);
        m_pString[dwLength] = 0;
    }
}

UnicodeString::~UnicodeString() throw()
{
    if (m_pString != NULL)
    {
        delete [] m_pString;
    }
}

bool UnicodeString::IsEmpty() const throw()
{
    return m_pString == NULL || m_pString[0] == 0;
}

void UnicodeString::Clear() throw()
{
    if (m_pString != NULL)
    {
        m_pString[0] = 0;
    }
}

unsigned long UnicodeString::GetLength() const throw()
{
    return m_pString != NULL ? StringLength(m_pString) : 0;
}

unsigned long UnicodeString::GetCapacity() const throw()
{
    return m_dwBufferSize;
}

bool UnicodeString::SetCapacity(unsigned long dwSize,bool bKeepData) throw()
{
    bool bRet(false);
    if (dwSize > m_dwBufferSize)
    {
        wchar_t * pNewData = new wchar_t[dwSize];
        if (pNewData != NULL)
        {
            if (bKeepData == true)
            {
                const unsigned long dwLength = GetLength();
                StringCopy(pNewData,m_pString,dwLength);
                pNewData[dwLength] = 0;
            }
            else
            {
                pNewData[0] = 0;
            }
            if (m_pString != NULL)
            {
                delete [] m_pString;
            }
            m_pString = pNewData;
            m_dwBufferSize = dwSize;
            bRet = true;
        }
    }
    else
    {
        bRet = true;
    }
    return bRet;
}

const wchar_t * UnicodeString::GetString() const throw()
{
    return m_pString;
}

wchar_t * UnicodeString::GetBuffer() throw()
{
    return m_pString;
}

wchar_t * UnicodeString::GetBuffer(unsigned long dwBufferSize) throw()
{
    if (SetCapacity(dwBufferSize,true) == false)
    {
        return NULL;
    }
    else
    {
        return m_pString;
    }
}

bool UnicodeString::SetAt(unsigned long dwIndex,wchar_t ch) throw()
{
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        m_pString[dwIndex] = ch;
        return true;
    }
    else
    {
        return false;
    }
}

bool UnicodeString::GetAt(unsigned long dwIndex,wchar_t & chRef) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        chRef = m_pString[dwIndex];
        return true;
    }
    else
    {
        return false;
    }
}

bool UnicodeString::Append(wchar_t ch) throw()
{
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwBufferSize,true) == false)
            {
                return false;
            }
        }
        m_pString[dwLength] = ch;
        m_pString[dwNewLength] = 0;
        return true;
    }
    else
    {
        return false;
    }
}

bool UnicodeString::Append(wchar_t ch,unsigned long dwCount) throw()
{
    bool bRet(false);
    if (ch != 0 && dwCount != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwCount;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        for (unsigned long i = 0; i < dwCount; ++i)
        {
            m_pString[dwLength + i] = ch;
        }
        bRet = true;
    }
    return bRet;
}

bool UnicodeString::Append(const wchar_t * pString) throw()
{
     bool bRet(false);
     const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
     if (dwAddLength != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwAddLength;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
             if (SetCapacity(dwAlloc,true) == false)
             {
                 return bRet;
             }
         }
         StringCopy(&m_pString[dwLength],pString,dwAddLength);
         m_pString[dwNewLength] = 0;
         bRet = true;
     }
     return bRet;
}

bool UnicodeString::Append(const wchar_t * pString,unsigned long dwwchar_tCount) throw()
{
     bool bRet(false);
     if (pString != NULL)
     {
         const unsigned long dwCopy = dwwchar_tCount != 0xFFFFFFFF ? dwwchar_tCount : StringLength(pString);
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwCopy;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
             if (SetCapacity(dwBufferSize,true) == false)
             {
                 return bRet;
             }
         }
         StringCopy(&m_pString[dwLength],pString,dwCopy);
         m_pString[dwNewLength] = 0;
         bRet = true;
     }
     return bRet;
}

bool UnicodeString::Append(const UnicodeString & str) throw()
{
    bool bRet(false);
    const unsigned long dwAddLength = str.GetLength();
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool UnicodeString::Append(const UnicodeString & str,unsigned long dwwchar_tCount) throw()
{
    bool bRet(false);
    const unsigned long dwTempAddLen = str.GetLength();
    unsigned long dwAddLength(0);
    if (dwwchar_tCount == 0 || dwwchar_tCount >= dwTempAddLen)
    {
        dwAddLength = dwTempAddLen;
    }
    else
    {
        dwAddLength = dwwchar_tCount;
    }
    
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool UnicodeString::Append(const UnicodeString * pStrings,unsigned long dwStringCount) throw()
{
    bool bRet(false);
    if (pStrings != NULL && dwStringCount != 0)
    {
        unsigned long * pdwLengths = new unsigned long[dwStringCount];
        if (pdwLengths != NULL)
        {
            unsigned long dwAddLength(0);
            for (unsigned long x = 0; x < dwStringCount; ++x)
            {
                pdwLengths[x] = pStrings[x].GetLength();
                dwAddLength += pdwLengths[x];
            }
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwAddLength + dwLength;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    delete [] pdwLengths;
                    return bRet;
                }
            }
            unsigned long dwTempPos(0);
            for (unsigned long v = 0; v < dwStringCount; ++v)
            {
                StringCopy(&m_pString[dwLength + dwTempPos],pStrings[v].m_pString,pdwLengths[v]);
                dwTempPos += pdwLengths[x];
            }
            m_pString[dwNewLength] = 0;
            delete [] pdwLengths;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(wchar_t ch) throw()
{
    bool bRet(false);
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength == 0)
        {
            m_pString[0] = ch;
            m_pString[1] = 0;
            bRet = true;
        }
        else
        {
            const unsigned long dwNewLength = dwLength + 1;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
                wchar_t * pNewData = new wchar_t[dwAllocate];
                if (pNewData != NULL)
                {
                    pNewData[0] = ch;
                    StringCopy(&pNewData[1],m_pString,dwLength);
                    pNewData[dwNewLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
            }
            else
            {
                for (unsigned long x = dwLength; x > 0; --x)
                {
                    m_pString[x] = m_pString[x - 1];
                }
                m_pString[0] = ch;
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(wchar_t ch,unsigned long dwCount) throw()
{
     bool bRet(false);
     if (ch != 0 && dwCount != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwCount;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
             wchar_t * pNewData = new wchar_t[dwAllocate];
             if (pNewData != NULL)
             {
                 unsigned long x(0);
                 for (; x < dwCount; ++x)
                 {
                     pNewData[x] = ch;
                 }
                 for (unsigned long y = 0; y < dwLength; ++y,++x)
                 {
                     pNewData[x] = m_pString[y];
                 }
                 pNewData[dwNewLength] = 0;
                 if (m_pString != NULL)
                 {
                     delete [] m_pString;
                 }
                 m_pString = pNewData;
                 m_dwBufferSize = dwAllocate;
                 bRet = true;
             }
         }
         else
         {
             for (unsigned long i = dwNewLength,j = dwLength; j > 0; --i,--j)
             {
                 m_pString[i - 1] = m_pString[j - 1];
             }
             for (unsigned long x = 0; x < dwCount; ++x)
             {
                 m_pString[x] = ch;
             }
             m_pString[dwNewLength] = 0;
             bRet = true;
         }
     }
     return bRet;
}

bool UnicodeString::Prepend(const wchar_t * pString) throw()
{
    bool bRet(false);
    const unsigned long dwPrependLen = pString != NULL ? StringLength(pString) : 0;
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
            wchar_t * pNewData = new wchar_t[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwPrependLen; ++x)
                {
                    pNewData[x] = pString[x];
                }
                for (unsigned long i = 0; i < dwLength; ++i,++x)
                {
                    pNewData[x] = m_pString[i];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long v = dwNewLength,z = dwLength; z > 0; --z,--v)
            {
                m_pString[v - 1] = m_pString[z - 1];
            }
            for (unsigned long t = 0; t < dwPrependLen; ++t)
            {
                m_pString[t] = pString[t];
            }
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(const wchar_t * pString,unsigned long dwwchar_tCount) throw()
{
    bool bRet(false);
    if (pString != NULL)
    {
        unsigned long dwPrependLen(0);
        if (dwwchar_tCount == 0 || dwwchar_tCount == 0xFFFFFFFF)
        {
            dwPrependLen = StringLength(pString);
        }
        else
        {
            dwPrependLen = dwwchar_tCount;
        }
        if (dwPrependLen != 0)
        {
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwLength + dwPrependLen;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
                wchar_t * pNewData = new wchar_t[dwAllocate];
                if (pNewData != NULL)
                {
                    unsigned long x(0);
                    for (; x < dwPrependLen; ++x)
                    {
                        pNewData[x] = pString[x];
                    }
                    for (unsigned long y = 0; y < dwLength; ++y,++x)
                    {
                        pNewData[x] = pString[y];
                    }
                    pNewData[dwNewLength] = 0;
                    if (m_pString != NULL)
                    {
                        delete [] m_pString;
                    }
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
            }
            else
            {
                for (unsigned long z = dwNewLength,v = dwLength; v > 0; --z,--v)
                {
                    m_pString[z - 1] = m_pString[v - 1];
                }
                for (unsigned long t = 0; t < dwNewLength; ++t)
                {
                    m_pString[t] = pString[t];
                }
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(const UnicodeString & str) throw()
{
    bool bRet(false);
    const unsigned long dwPrependLen = str.GetLength();
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
            wchar_t * pNewData = new wchar_t[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwPrependLen; ++x)
                {
                    pNewData[x] = str.m_pString[x];
                }
                for (unsigned long i = 0; i < dwLength; ++i,++x)
                {
                    pNewData[x] = m_pString[i];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long z = dwNewLength,v = dwLength; v > 0; --v,--z)
            {
                m_pString[z - 1] = m_pString[v - 1];
            }
            for (unsigned long t = 0; t < dwPrependLen; ++t)
            {
                m_pString[t] = str.m_pString[t];
            }
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(const UnicodeString & str,unsigned long dwwchar_tCount) throw()
{
    bool bRet(false);
    const unsigned long dwTempLen = str.GetLength();
    unsigned long dwPrependLen(0);
    if (dwwchar_tCount == 0 || dwwchar_tCount > dwTempLen)
    {
        dwPrependLen = dwTempLen;
    }
    else
    {
        dwPrependLen = dwwchar_tCount;
    }
    
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewLength);
        const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
        wchar_t * pNewData = new wchar_t[dwAlloc];
        if (pNewData != NULL)
        {
            StringCopy(pNewData,str.m_pString,dwPrependLen);
            StringCopy(&pNewData[dwPrependLen],m_pString,dwLength);
            pNewData[dwNewLength] = 0;
            if (m_pString != NULL)
            {
                delete [] m_pString;
            }
            m_pString = pNewData;
            m_dwBufferSize = dwAlloc;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Prepend(const UnicodeString * pStrings,unsigned long dwStringCount) throw()
{
    bool bRet(false);
    if (pStrings != NULL && dwStringCount != 0)
    {
        unsigned long * pdwLengths = new unsigned long[dwStringCount];
        if (pdwLengths != NULL)
        {
            unsigned long dwPrependLen(0);
            for (unsigned long x = 0; x < dwStringCount; ++x)
            {
                pdwLengths[x] = pStrings[x].GetLength();
                dwPrependLen += pdwLengths[x];
            }
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwLength + dwPrependLen;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
                wchar_t * pNewData = new wchar_t[dwAllocate];
                if (pNewData != NULL)
                {
                    unsigned long xt(0);
                    for (unsigned long v = 0; v < dwStringCount; ++v)
                    {
                        StringCopy(&pNewData[xt],pStrings[v].m_pString,pdwLengths[v]);
                        xt += pdwLengths[v];
                    }
                    StringCopy(&pNewData[xt],m_pString,dwLength);
                    pNewData[dwNewLength] = 0;
                    if (m_pString != NULL)
                    {
                        delete [] m_pString;
                    }
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
                else
                {
                    delete [] pdwLengths;
                    return bRet;
                }
            }
            else
            {
                for (unsigned long z = dwNewLength,v = dwLength; v > 0; --v,--z)
                {
                    m_pString[z - 1] = m_pString[v - 1];
                }
                unsigned long vx(0);
                for (unsigned long it = 0; it < dwStringCount; ++it)
                {
                    StringCopy(&m_pString[vx],pStrings[it].m_pString,pdwLengths[it]);
                    vx += pdwLengths[it];
                }
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
            delete [] pdwLengths;
        }
    }
    return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,wchar_t ch) throw()
{
    bool bRet(false);
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            wchar_t * pNewData = new wchar_t[dwAlloc];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwIndex; ++x)
                {
                    pNewData[x] = m_pString[x];
                }
                unsigned long y = x++;
                pNewData[y] = ch;
                for (; y < dwLength; ++y,++x)
                {
                    pNewData[x] = m_pString[y];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long v = dwLength; v > dwIndex; --v)
            {
                m_pString[v] = m_pString[v - 1];
            }
            m_pString[dwIndex] = ch;
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,wchar_t ch,unsigned long dwCount) throw()
{
    bool bRet(false);
    if (ch != 0 && dwCount != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwCount;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
            wchar_t * pNewData = new wchar_t[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwIndex; ++x)
                {
                    pNewData[x] = m_pString[x];
                }
                unsigned long y = x++;
                pNewData[y] = ch;
                for (; y < dwLength; ++y,++x)
                {
                    pNewData[x] = m_pString[y];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            if (dwIndex >= dwLength)
            {
                for (unsigned long vi = 0; vi < dwCount; ++vi)
                {
                    m_pString[dwLength + vi] = ch;
                }
                m_pString[dwNewLength] = 0;
            }
            else
            {
                for (unsigned long zt = dwNewLength,zu = dwLength; zu > dwIndex; --zu,--zt)
                {
                    m_pString[zt - 1] = m_pString[zu - 1];
                }
                for (unsigned long zx = 0; zx < dwCount; ++zx)
                {
                    m_pString[dwIndex + zx] = ch;
                }
                m_pString[dwNewLength] = 0;
            }
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,const wchar_t * pString) throw()
{
     bool bRet(false);
     const unsigned long dwInsertLen = pString != NULL ? StringLength(pString) : 0;
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwNewLength);
             wchar_t * pNewData = new wchar_t[dwAllocate];
             if (pNewData != NULL)
             {
                 unsigned long x = 0;
                 for (; x < dwIndex; ++x)
                 {
                     pNewData[x] = m_pString[x];
                 }
                 unsigned long y = x;
                 for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
                 {
                     pNewData[x] = pString[i];
                 }
                 for (; y < dwLength; ++y,++x)
                 {
                     pNewData[x] = m_pString[y];
                 }
                 pNewData[dwNewLength] = 0;
                 if (m_pString != NULL)
                 {
                     delete [] m_pString;
                 }
                 m_pString = pNewData;
                 m_dwBufferSize = dwAllocate;
                 bRet = true;
             }
         }
         else
         {
             if (dwIndex >= dwLength)
             {
                 for (unsigned long xt = 0; xt < dwInsertLen; ++xt)
                 {
                     m_pString[dwLength + xt] = pString[xt];
                 }
                 m_pString[dwNewLength] = 0;
             }
             else
             {
                 for (unsigned long zt = dwNewLength,zu = dwLength; zu > dwIndex; --zt,--zu)
                 {
                     m_pString[zt - 1] = m_pString[zu - 1];
                 }
                 for (unsigned long zd = 0; zd < dwInsertLen; ++zd)
                 {
                     m_pString[dwIndex + zd] = pString[zd];
                 }
                 m_pString[dwNewLength] = 0;
             }
             bRet = true;
         }
     }
     return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,const wchar_t * pString,unsigned long dwwchar_tCount) throw()
{
     bool bRet(false);
     if (pString != NULL)
     {
         unsigned long dwInsertLen(0);
         if (dwwchar_tCount == 0 || dwwchar_tCount == 0xFFFFFFFF)
         {
             dwInsertLen = StringLength(pString);
         }
         else
         {
             dwInsertLen = dwwchar_tCount;
         }
         
         if (dwInsertLen != 0)
         {
             const unsigned long dwLength = GetLength();
             const unsigned long dwNewLength = dwLength + dwInsertLen;
             if (dwNewLength >= m_dwBufferSize)
             {
                 const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
                 wchar_t * pNewData = new wchar_t[dwBufferSize];
                 if (pNewData != NULL)
                 {
                     unsigned long x = 0;
                     for (; x < dwIndex; ++x)
                     {
                         pNewData[x] = m_pString[x];
                     }
                     unsigned long y = x;
                     for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
                     {
                         pNewData[x] = pString[i];
                     }
                     for (; y < dwLength; ++y,++x)
                     {
                         pNewData[x] = m_pString[y];
                     }
                     pNewData[dwNewLength] = 0;
                     if (m_pString != NULL)
                     {
                         delete [] m_pString;
                     }
                     m_pString = pNewData;
                     m_dwBufferSize = dwBufferSize;
                     bRet = true;
                 }
             }
             else
             {
                 if (dwIndex >= dwLength)
                 {
                     for (unsigned long x = 0; x < dwInsertLen; ++x)
                     {
                         m_pString[dwLength + x] = pString[x];
                     }
                     m_pString[dwNewLength] = 0;
                 }
                 else
                 {
                     for (unsigned long zt = dwNewLength, zu = dwLength; zu > dwIndex; --zu,--zt)
                     {
                         m_pString[zt - 1] = m_pString[zu - 1];
                     }
                     for (unsigned long xt = 0; xt < dwInsertLen; ++xt)
                     {
                         m_pString[dwIndex + xt] = pString[xt];
                     }
                     m_pString[dwNewLength] = 0;
                 }
                 bRet = true;
             }
         }
     }
     return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,const UnicodeString & str) throw()
{
     bool bRet(false);
     const unsigned long dwInsertLen = str.GetLength();
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewLength);
         const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
         wchar_t * pNewData = new wchar_t[dwAlloc];
         if (pNewData != NULL)
         {
             unsigned long x = 0;
             for (; x < dwIndex; ++x)
             {
                 pNewData[x] = m_pString[x];
             }
             unsigned long y = x;
             for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
             {
                 pNewData[x] = str.m_pString[i];
             }
             for (; y < dwLength; ++y,++x)
             {
                 pNewData[x] = m_pString[y];
             }
             pNewData[dwNewLength] = 0;
             if (m_pString != NULL)
             {
                 delete [] m_pString;
             }
             m_pString = pNewData;
             m_dwBufferSize = dwAlloc;
             bRet = true;
         }
     }
     return bRet;
}

bool UnicodeString::Insert(unsigned long dwIndex,const UnicodeString & str,unsigned long dwwchar_tCount) throw()
{
     bool bRet(false);
     const unsigned long dwTempLen = str.GetLength();
     unsigned long dwInsertLen(0);
     if (dwwchar_tCount == 0 || dwwchar_tCount > dwTempLen)
     {
         dwInsertLen = dwTempLen;
     }
     else
     {
         dwInsertLen = dwwchar_tCount;
     }
     
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewLength);
         const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
         wchar_t * pNewData = new wchar_t[dwAlloc];
         if (pNewData != NULL)
         {
             unsigned long x = 0;
             for (; x < dwIndex; ++x)
             {
                 pNewData[x] = m_pString[x];
             }
             unsigned long y = x;
             for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
             {
                 pNewData[x] = str.m_pString[i];
             }
             for (; y < dwLength; ++y,++x)
             {
                 pNewData[x] = m_pString[y];
             }
             pNewData[dwNewLength] = 0;
             if (m_pString != NULL)
             {
                 delete [] m_pString;
             }
             m_pString = pNewData;
             m_dwBufferSize = dwAlloc;
             bRet = true;
         }
     }
     return bRet;
}

bool UnicodeString::Find(wchar_t chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long x = 0; x < dwLength; ++x)
        {
            if (m_pString[x] == chFind)
            {
                dwFoundIndex = x;
                bFound = true;
                break;
            }
        }
    }
    return bFound;
}

bool UnicodeString::Find(const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLen <= dwLength)
        {
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLen;
            
            for (unsigned long x = 0; x <= dwLastPossibleIndex; ++x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLen; ++i)
                {
                    if (m_pString[x + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::Find(const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            for (unsigned long x = 0; x <= dwLastPossibleIndex; ++x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                   if (m_pString[x + i] != strFind.m_pString[i])
                   {
                       bFound = false;
                       break;
                   }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindFrom(unsigned long dwStart,wchar_t chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (m_pString[x] == chFind)
                {
                    dwFoundIndex = x;
                    bFound = true;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindFrom(unsigned long dwStart,const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            const unsigned long dwCheckIndex = dwStart + dwFindLen;
            if (dwCheckIndex <= dwLength)
            {
                const unsigned long dwLastPossibleIndex = dwLength - dwFindLen;
                for (unsigned long x = dwStart; x <= dwLastPossibleIndex; ++x)
                {
                    bFound = true;
                    for (unsigned long i = 0; i < dwFindLen; ++i)
                    {
                        if (m_pString[x + i] != pstrFind[i])
                        {
                            bFound = false;
                            break;
                        }
                    }
                    
                    if (bFound == true)
                    {
                        dwFoundIndex = x;
                        break;
                    }
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindFrom(unsigned long dwStart,const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            const unsigned long dwCheckIndex = dwStart + dwFindLength;
            if (dwCheckIndex <= dwLength)
            {
                const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
                for (unsigned long x = dwStart; x <= dwLastPossibleIndex; ++x)
                {
                    bFound = true;
                    for (unsigned long i = 0; i < dwFindLength; ++i)
                    {
                        if (m_pString[x + i] != strFind.m_pString[i])
                        {
                            bFound = false;
                            break;
                        }
                    }
                    
                    if (bFound == true)
                    {
                        dwFoundIndex = x;
                        break;
                    }
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindLastInstanceOf(wchar_t chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long x = dwLength; x > 0; --x)
        {
            if (m_pString[x - 1] == chFind)
            {
                dwFoundIndex = x - 1;
                bFound = true;
                break;
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindLastInstanceOf(const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLen <= dwLength)
        {
            const unsigned long dwFirstPossibleIndex = dwLength - dwFindLen;
            for (unsigned long x = dwFirstPossibleIndex; x < dwLength; --x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLen; ++i)
                {
                    if (m_pString[x + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindLastInstanceOf(const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            const unsigned long dwFirstPossibleIndex = dwLength - dwFindLength;
            for (unsigned long x = dwFirstPossibleIndex; x < dwLength; --x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[x + i] != strFind.m_pString[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

void UnicodeString::FindAllInstancesOf(wchar_t chFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            if (m_pString[i] == chFind)
            {
                arFoundIndices.push_back(i);
            }
        }
    }
}

void UnicodeString::FindAllInstancesOf(const wchar_t * pstrFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    const unsigned long dwFindLength = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            bool bFound = false;
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            unsigned long dwCurrentIndex(0);
            do
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[dwCurrentIndex + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    arFoundIndices.push_back(dwCurrentIndex);
                    dwCurrentIndex += dwFindLength;
                }
                else
                {
                    ++dwCurrentIndex;
                }
            } while (dwCurrentIndex <= dwLastPossibleIndex);
        }
    }
}

void UnicodeString::FindAllInstancesOf(const UnicodeString & strFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            bool bFound = false;
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            unsigned long dwCurrentIndex(0);
            do
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[dwCurrentIndex + i] != strFind.m_pString[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    arFoundIndices.push_back(dwCurrentIndex);
                    dwCurrentIndex += dwFindLength;
                }
                else
                {
                    ++dwCurrentIndex;
                }
            } while (dwCurrentIndex <= dwLastPossibleIndex);
        }
    }
}

bool UnicodeString::FindOneOf(const wchar_t * pstrwchar_tSet,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const UnicodeString strTemp(pstrwchar_tSet);
    const unsigned long dwwchar_tSetLen = strTemp.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        unsigned long dwTempIndex(0);
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            bFound = strTemp.Find(m_pString[i],dwTempIndex);
            if (bFound == true)
            {
                dwFoundIndex = i;
                break;
            }
        }
    }
    return bFound;
}

bool UnicodeString::FindOneOf(const UnicodeString & strwchar_tSet,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwwchar_tSetLen = strwchar_tSet.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        unsigned long dwTempIndex(0);
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            bFound = strwchar_tSet.Find(m_pString[i],dwTempIndex);
            if (bFound == true)
            {
                dwFoundIndex = i;
                break;
            }
        }
    }
    return bFound;
}

unsigned long UnicodeString::Replace(wchar_t chOld,wchar_t chNew) throw()
{
    unsigned long dwReplacedCount(0);
    if (chOld != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            if (chNew != 0)
            {
                for (unsigned long i = 0; i < dwLength; ++i)
                {
                    if (m_pString[i] == chOld)
                    {
                        m_pString[i] = chNew;
                        ++dwReplacedCount;
                    }
                }
            }
            else
            {
                wchar_t * pNewData = new wchar_t[m_dwBufferSize];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0);
                    for (unsigned long x = 0; x < dwLength; ++x)
                    {
                        if (m_pString[x] != chOld)
                        {
                            pNewData[dwBufferPos] = m_pString[x];
                            ++dwBufferPos;
                        }
                        else
                        {
                            ++dwReplacedCount;
                        }
                    }
                    pNewData[dwBufferPos] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                }
            }
        }
    }
    return dwReplacedCount;
}

unsigned long UnicodeString::Replace(const wchar_t * pstrOld,const wchar_t * pstrNew) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = pstrOld != NULL ? StringLength(pstrOld) : 0;
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(pstrOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = pstrNew != NULL ? StringLength(pstrNew) : 0;
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwFoundCount;
                bSubtract = true;
            }

            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }

            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                wchar_t * pNewData = new wchar_t[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    for (unsigned long i = 0; i < dwFoundCount; ++i)
                    {
                        for (; dwCurrentPos < arFoundIndices[i]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = pstrNew[v];
                        }
                        dwCurrentPos += dwOldLength;
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = pstrNew[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long UnicodeString::Replace(const UnicodeString & strOld,const UnicodeString & strNew) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = strOld.GetLength();
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(strOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = strNew.GetLength();
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwAdjustment;
                bSubtract = true;
            }
            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }
            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                wchar_t * pNewData = new wchar_t[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    for (unsigned long t = 0; t < dwFoundCount; ++t)
                    {
                        for (; dwCurrentPos < arFoundIndices[t]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = strNew.m_pString[v];
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = strNew.m_pString[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long UnicodeString::Replace(const wchar_t * pstrOld,const wchar_t * pstrNew,bool bWholeWordOnly) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = pstrOld != NULL ? StringLength(pstrOld) : 0;
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(pstrOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = pstrNew != NULL ? StringLength(pstrNew) : 0;
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwFoundCount;
                bSubtract = true;
            }

            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }

            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                wchar_t * pNewData = new wchar_t[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    bool bReplace = false;
                    for (unsigned long i = 0; i < dwFoundCount; ++i)
                    {
                        bReplace = false;
                        for (; dwCurrentPos < arFoundIndices[i]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        if (bWholeWordOnly == true)
                        {
                            if (dwCurrentPos != 0)
                            {
                                if (IsCharAlphaDigit(m_pString[dwCurrentPos - 1]) == false && m_pString[dwCurrentPos - 1] != '_')
                                {
                                    bReplace = true;
                                }
                            }
                            else if (IsCharAlphaDigit(m_pString[dwCurrentPos]) == false && m_pString[dwCurrentPos] != '_')
                            {
                                bReplace = true;
                            }
                        }
                        if (bReplace == true)
                        {
                            if (IsCharAlphaDigit(m_pString[dwCurrentPos + dwOldLength]) == true && m_pString[dwCurrentPos + dwOldLength] == '_')
                            {
                                bReplace = false;
                            }
                        }

                        if (bReplace == true)
                        {
                            for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = pstrNew[v];
                            }
                            dwCurrentPos += dwOldLength;
                        }
                        else
                        {
                            for (unsigned long it = 0; it < dwOldLength; ++it,++dwCurrentPos,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                            }
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = pstrNew[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long UnicodeString::Replace(const UnicodeString & strOld,const UnicodeString & strNew,bool bWholeWordOnly) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = strOld.GetLength();
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(strOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = strNew.GetLength();
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwAdjustment;
                bSubtract = true;
            }
            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }
            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                wchar_t * pNewData = new wchar_t[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    bool bReplace = false;
                    for (unsigned long t = 0; t < dwFoundCount; ++t)
                    {
                        bReplace = false;
                        for (; dwCurrentPos < arFoundIndices[t]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        if (bWholeWordOnly == true)
                        {
                            if (dwCurrentPos != 0)
                            {
                                if (IsCharAlphaDigit(m_pString[dwCurrentPos - 1]) == false && m_pString[dwCurrentPos - 1] != '_')
                                {
                                    bReplace = true;
                                }
                            }
                            else if (IsCharAlphaDigit(m_pString[dwCurrentPos]) == false && m_pString[dwCurrentPos] != '_')
                            {
                                bReplace = true;
                            }
                        }
                        if (bReplace == true)
                        {
                            if (IsCharAlphaDigit(m_pString[dwCurrentPos + dwOldLength]) == true && m_pString[dwCurrentPos + dwOldLength] == '_')
                            {
                                bReplace = false;
                            }
                        }

                        if (bReplace == true)
                        {
                            for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = strNew.m_pString[v];
                            }
                            dwCurrentPos += dwOldLength;
                        }
                        else
                        {
                            for (unsigned long it = 0; it < dwOldLength; ++it,++dwCurrentPos,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                            }
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = strNew.m_pString[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

bool UnicodeString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,wchar_t ch) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0),dwNewLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
        }
        else
        {
            dwOldLength = dwCount;
        }
        dwNewLength = ch != 0 ? 1 : 0;
        for (unsigned long i = dwIndex,j = dwIndex + dwOldLength; j < dwLength; ++j,++i)
        {
            m_pString[i + dwNewLength] = m_pString[j];
        }
        if (dwNewLength != 0)
        {
            m_pString[dwIndex] = ch;
        }
        m_pString[dwLength + dwNewLength - dwOldLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool UnicodeString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const wchar_t * pString) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwNewLength = pString != NULL ? StringLength(pString) : 0;
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
            const unsigned long dwTotalLength = dwIndex + dwNewLength;
            if (dwTotalLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwTotalLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    return false;
                }
            }
            for (unsigned long i = 0; i < dwNewLength; ++i)
            {
                m_pString[dwIndex + i] = pString[i];
            }
            m_pString[dwTotalLength] = 0;
            bRet = true;
        }
        else
        {
            dwOldLength = dwCount;
            unsigned long dwAdjustment(0);
            bool bSubtract(false);
            if (dwOldLength == dwNewLength)
            {
                for (unsigned long vx = 0; vx < dwNewLength; ++vx)
                {
                    m_pString[dwIndex + vx] = pString[vx];
                }
                return true;
            }
            else if (dwOldLength < dwNewLength)
            {
                dwAdjustment = dwNewLength - dwOldLength;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = dwOldLength - dwNewLength;
                bSubtract = true;
            }
            unsigned long dwTotalLength(0);
            if (bSubtract == true)
            {
                dwTotalLength = dwLength - dwAdjustment;
            }
            else
            {
                dwTotalLength = dwLength + dwAdjustment;
            }
            
            const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwTotalLength);
            const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
            wchar_t * pTempData = new wchar_t[dwAlloc];
            if (pTempData != NULL)
            {
                unsigned long dwBufferPos(0),dwCurrentPos(0);
                for (; dwCurrentPos < dwIndex; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                dwCurrentPos += dwOldLength;
                for (unsigned long it = 0; it < dwNewLength; ++it,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = pString[it];
                }
                for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                pTempData[dwTotalLength] = 0;
                delete [] m_pString;
                m_pString = pTempData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool UnicodeString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const UnicodeString & str) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwNewLength = str.GetLength();
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
            const unsigned long dwTotalLength = dwIndex + dwNewLength;
            if (dwTotalLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwTotalLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    return false;
                }
            }
            for (unsigned long i = 0; i < dwNewLength; ++i)
            {
                m_pString[dwIndex + i] = str.m_pString[i];
            }
            m_pString[dwTotalLength] = 0;
            bRet = true;
        }
        else
        {
            dwOldLength = dwCount;
            unsigned long dwAdjustment(0);
            bool bSubtract(false);
            if (dwOldLength == dwNewLength)
            {
                for (unsigned long vx = 0; vx < dwNewLength; ++vx)
                {
                    m_pString[dwIndex + vx] = str.m_pString[vx];
                }
                return true;
            }
            else if (dwOldLength < dwNewLength)
            {
                dwAdjustment = dwNewLength - dwOldLength;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = dwOldLength - dwNewLength;
                bSubtract = true;
            }
            unsigned long dwTotalLength(0);
            if (bSubtract == true)
            {
                dwTotalLength = dwLength - dwAdjustment;
            }
            else
            {
                dwTotalLength = dwLength + dwAdjustment;
            }
            
            const unsigned long dwTempAlloc = AnsiString::DetermineBufferSize(dwTotalLength);
            const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
            wchar_t * pTempData = new wchar_t[dwAlloc];
            if (pTempData != NULL)
            {
                unsigned long dwBufferPos(0),dwCurrentPos(0);
                for (; dwCurrentPos < dwIndex; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                dwCurrentPos += dwOldLength;
                for (unsigned long it = 0; it < dwNewLength; ++it,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = str.m_pString[it];
                }
                for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                pTempData[dwTotalLength] = 0;
                delete [] m_pString;
                m_pString = pTempData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
    }
    return bRet;
}

unsigned long UnicodeString::Remove(wchar_t chRemove) throw()
{
    return Replace(chRemove,0);
}

unsigned long UnicodeString::Remove(const wchar_t * pstrRemove) throw()
{
    return Replace(pstrRemove,NULL);
}

unsigned long UnicodeString::Remove(const UnicodeString & strRemove) throw()
{
    return Replace(strRemove,UnicodeString());
}

unsigned long UnicodeString::Remove(const wchar_t * pstrRemove,bool bWholeWordOnly) throw()
{
    return Replace(pstrRemove,NULL,bWholeWordOnly);
}

unsigned long UnicodeString::Remove(const UnicodeString & strRemove,bool bWholeWordOnly) throw()
{
    return Replace(strRemove,UnicodeString(),bWholeWordOnly);
}

bool UnicodeString::RemoveAt(unsigned long dwIndex,unsigned long dwCount) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            m_pString[dwIndex] = 0;
            bRet = true;
        }
        else
        {
            for (unsigned long i = dwIndex,j = dwIndex + dwCount; j < dwLength; ++j,++i)
            {
                m_pString[i] = m_pString[j];
            }
            m_pString[dwLength - dwCount] = 0;
            bRet = true;
        }
    }
    return bRet;
}

void UnicodeString::MakeUppercase() throw()
{
    const wchar_t chDif = L'A' - L'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= L'a' && m_pString[i] <= L'z')
        {
            m_pString[i] += chDif;
        }
    }
}

void UnicodeString::MakeLowercase() throw()
{
    const wchar_t chDif = L'A' - L'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= L'A' && m_pString[i] <= L'Z')
        {
            m_pString[i] -= chDif;
        }
    }
}

void UnicodeString::SwitchCase() throw()
{
    const wchar_t chDif = L'A' - L'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= L'A' && m_pString[i] <= L'Z')
        {
            m_pString[i] -= chDif;
        }
        else if (m_pString[i] >= L'a' && m_pString[i] <= L'z')
        {
            m_pString[i] += chDif;
        }
    }
}

void UnicodeString::Reverse() throw()
{
    const unsigned long dwLength = GetLength();
    if (dwLength != 0)
    {
        wchar_t chTemp(0);
        for (unsigned long x = 0,y = dwLength - 1; x < y; ++x,--y)
        {
            chTemp = m_pString[x];
            m_pString[x] = m_pString[y];
            m_pString[y] = chTemp;
        }
    }
}

bool UnicodeString::SubString(unsigned long dwStart,unsigned long dwCount,UnicodeString & strBuffer) const throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwStart;
        unsigned long dwCopyLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwCopyLength = dwRemainingLength;
        }
        else
        {
            dwCopyLength = dwCount;
        }
        const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCopyLength);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCopyLength);
            strBuffer.m_pString[dwCopyLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::Extract(unsigned long dwStart,unsigned long dwCount,UnicodeString & strBuffer) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwStart;
        unsigned long dwCopyLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwCopyLength = dwRemainingLength;
        }
        else
        {
            dwCopyLength = dwCount;
        }
        const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCopyLength);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCopyLength);
            strBuffer.m_pString[dwCopyLength] = 0;
            const unsigned long dwCheckIndex = dwStart + dwCopyLength;
            if (dwCheckIndex == dwLength)
            {
                m_pString[0] = 0;
            }
            else
            {
                for (unsigned long i = dwCheckIndex,j = 0; i < dwLength; ++i,++j)
                {
                    m_pString[j] = m_pString[i];
                }
                m_pString[dwLength - dwCopyLength] = 0;
            }
            bRet = true;
        }
    }
    return bRet;
}

bool UnicodeString::SpanIncluding(const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw()
{
    const UnicodeString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanIncluding(const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
            {
                break;
            }
        }
        const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanIncluding(unsigned long dwStart,const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const UnicodeString strCharSet(pstrCharSet);
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == true)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanIncluding(unsigned long dwStart,const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == true)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanExcluding(const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw()
{
    const UnicodeString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == true)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanExcluding(const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == true)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanExcluding(unsigned long dwStart,const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const UnicodeString strCharSet(pstrCharSet);
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == false)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool UnicodeString::SpanExcluding(unsigned long dwStart,const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == false)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = AnsiString::DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

void UnicodeString::Scramble() throw()
{
    const unsigned long dwLength = GetLength() * sizeof(wchar_t);
    if (dwLength != 0)
    {
        unsigned char * pData = reinterpret_cast<unsigned char *>(m_pString);
        const unsigned char ucBits1[8] = { 16,64,128,1,8,4,2,32 };
        const unsigned char ucBits2[8] = { 64,32,1,128,2,8,16,4 };
        unsigned char ucBit(0),ucTemp1(0),ucTemp2(0);
        unsigned long x(0);
        do
        {
            ucBit = 1;
            ucTemp1 = pData[x];
            pData[x] = 0;
            ucTemp2 = pData[x + 1];
            pData[x + 1] = 0;
            for (unsigned long i = 0; i < 8; ++i)
            {
                if (ucTemp1 & ucBit)
                {
                    pData[x] |= ucBits2[i];
                }
                
                if (ucTemp2 & ucBit)
                {
                    pData[x + 1] |= ucBits1[i];
                }

                ucBit <<= 1;
            }
            x += sizeof(wchar_t);
        } while (x < dwLength);
    }
}

void UnicodeString::Descramble() throw()
{
    const unsigned long dwLength = GetLength() * sizeof(wchar_t);
    if (dwLength != 0)
    {
        unsigned char * pData = reinterpret_cast<unsigned char *>(m_pString);
        const unsigned char ucBits1[8] = { 8,64,32,1,128,2,4 };
        const unsigned char ucBits2[8] = { 4,16,128,32,64,2,1,8 };
        unsigned char ucBit(0),ucTemp1(0),ucTemp2(0);
        unsigned long x(0);
        do
        {
            ucBit = 1;
            ucTemp1 = pData[x];
            pData[x] = 0;
            ucTemp2 = pData[x + 1];
            pData[x + 1] = 0;
            for (unsigned long i = 0; i < 8; ++i)
            {
                if (ucTemp1 & ucBit)
                {
                    pData[x] |= ucBits2[i];
                }
                
                if (ucTemp2 & ucBit)
                {
                    pData[x + 1] |= ucBits1[i];
                }

                ucBit <<= 1;
            }
            x += sizeof(wchar_t);
        } while (x < dwLength);
    }
}

void UnicodeString::TrimLeft(wchar_t chTrim) throw()
{
    if (chTrim != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (m_pString[dwCount] != chTrim)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void UnicodeString::TrimLeft(const wchar_t * pstrwchar_tSet) throw()
{
    const UnicodeString strwchar_tSet(pstrwchar_tSet);
    const unsigned long dwwchar_tSetLen = strwchar_tSet.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (strwchar_tSet.Find(m_pString[dwCount],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void UnicodeString::TrimLeft(const UnicodeString & strwchar_tSet) throw()
{
    const unsigned long dwwchar_tSetLen = strwchar_tSet.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (strwchar_tSet.Find(m_pString[dwCount],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void UnicodeString::TrimLeft() throw()
{
    TrimLeft(L" \t\r\n");
}

void UnicodeString::TrimRight(wchar_t chTrim) throw()
{
    if (chTrim != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (m_pString[v - 1] != chTrim)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void UnicodeString::TrimRight(const wchar_t * pstrwchar_tSet) throw()
{
    const UnicodeString strwchar_tSet(pstrwchar_tSet);
    const unsigned long dwwchar_tSetLen = strwchar_tSet.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (strwchar_tSet.Find(m_pString[v - 1],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void UnicodeString::TrimRight(const UnicodeString & strwchar_tSet) throw()
{
    const unsigned long dwwchar_tSetLen = strwchar_tSet.GetLength();
    if (dwwchar_tSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (strwchar_tSet.Find(m_pString[v - 1],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void UnicodeString::TrimRight() throw()
{
    TrimRight(L" \t\r\n");
}

AnsiString UnicodeString::ToAnsi() const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwLength);
    AnsiString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        const long nWritten = ::WideCharToMultiByte(CP_ACP,WC_COMPOSITECHECK,m_pString,dwLength,strRetValue.m_pString,dwBufferSize,NULL,NULL);
        strRetValue.m_pString[nWritten] = 0;
    }
    return strRetValue;
}

const UnicodeString & UnicodeString::operator = (wchar_t ch) throw()
{
    if (m_pString != NULL)
    {
        m_pString[0] = ch;
        m_pString[1] = 0;
    }
    else
    {
        if (SetCapacity(16,false) == true)
        {
            m_pString[0] = ch;
            m_pString[1] = 0;
        }
    }
    return *this;
}

const UnicodeString & UnicodeString::operator = (const wchar_t * pString) throw()
{
    const unsigned long dwLength = pString != NULL ? StringLength(pString) : 0;
    if (dwLength >= m_dwBufferSize)
    {
        const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwLength);
        if (SetCapacity(dwBufferSize,false) == false)
        {
            return *this;
        }
    }
    StringCopy(m_pString,pString,dwLength);
    m_pString[dwLength] = 0;
    return *this;
}

const UnicodeString & UnicodeString::operator = (const UnicodeString & str) throw()
{
    if (this != &str)
    {
        const unsigned long dwNewLength = str.GetLength();
        const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
        if (SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(m_pString,str.m_pString,dwNewLength);
            m_pString[dwNewLength] = 0;
        }
    }
    return *this;
}

UnicodeString & UnicodeString::operator += (wchar_t ch) throw()
{
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwBufferSize,true) == false)
            {
                return *this;
            }
        }
        m_pString[dwLength] = ch;
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

UnicodeString & UnicodeString::operator += (const wchar_t * pString) throw()
{
    const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return *this;
            }
        }
        StringCopy(&m_pString[dwLength],pString,dwAddLength);
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

UnicodeString & UnicodeString::operator += (const UnicodeString & str) throw()
{
    const unsigned long dwAddLength = str.GetLength();
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = AnsiString::DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return *this;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

UnicodeString UnicodeString::operator + (wchar_t ch) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwNewLength = dwLength + 1;
    const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
    UnicodeString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        strRetValue.m_pString[dwLength] = ch;
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

UnicodeString UnicodeString::operator + (const wchar_t * pString) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwNewLength = dwLength + dwAddLength;
    const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
    UnicodeString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        StringCopy(&strRetValue.m_pString[dwLength],pString,dwAddLength);
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

UnicodeString UnicodeString::operator + (const UnicodeString & str) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwAddLength = str.GetLength();
    const unsigned long dwNewLength = dwLength + dwAddLength;
    const unsigned long dwBufferSize = AnsiString::DetermineBufferSize(dwNewLength);
    UnicodeString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        StringCopy(&strRetValue.m_pString[dwLength],str.m_pString,dwAddLength);
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

bool UnicodeString::operator == (wchar_t ch) const throw()
{
    if (ch == 0)
    {
        return m_pString == NULL || m_pString[0] == 0;
    }
    else
    {
        return m_pString != NULL && m_pString[0] == ch && m_pString[1] == 0;
    }
}

bool UnicodeString::operator == (const wchar_t * pString) const throw()
{
    const unsigned long dwCompareLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwLength = GetLength();
    if (dwLength == dwCompareLength)
    {
        bool bSame = true;
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            if (m_pString[i] != pString[i])
            {
                bSame = false;
                break;
            }
        }
        return bSame;
    }
    else
    {
        return false;
    }
}

bool UnicodeString::operator == (const UnicodeString & str) const throw()
{
    if (this != &str)
    {
        const unsigned long dwCompareLength = str.GetLength();
        const unsigned long dwLength = GetLength();
        if (dwCompareLength == dwLength)
        {
            bool bSame = true;
            for (unsigned long i = 0; i < dwLength; ++i)
            {
                if (m_pString[i] != str.m_pString[i])
                {
                    bSame = false;
                    break;
                }
            }
            return bSame;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return true;
    }
}

bool UnicodeString::operator != (wchar_t ch) const throw()
{
    if (ch == 0)
    {
        return m_pString != NULL && m_pString[0] != 0;
    }
    else
    {
        return m_pString == NULL || m_pString[0] != ch || m_pString[1] != 0;
    }
}

bool UnicodeString::operator != (const wchar_t * pString) const throw()
{
     const unsigned long dwCompareLength = pString != NULL ? StringLength(pString) : 0;
     const unsigned long dwLength = GetLength();
     if (dwCompareLength == dwLength)
     {
         bool bDifferent = false;
         for (unsigned long i = 0; i < dwLength; ++i)
         {
             if (m_pString[i] != pString[i])
             {
                 bDifferent = true;
                 break;
             }
         }
         return bDifferent;
     }
     else
     {
         return true;
     }
}

bool UnicodeString::operator != (const UnicodeString & str) const throw()
{
    if (this != &str)
    {
        const unsigned long dwCompareLength = str.GetLength();
        const unsigned long dwLength = GetLength();
        if (dwCompareLength != dwLength)
        {
            return true;
        }
        else
        {
            bool bDifferent = false;
            for (unsigned long x = 0; x < dwLength; ++x)
            {
                if (m_pString[x] != str.m_pString[x])
                {
                    bDifferent = true;
                    break;
                }
            }
            return bDifferent;
        }
    }
    else
    {
        return false;
    }
}

const wchar_t & UnicodeString::operator [] (unsigned long dwIndex) const throw()
{
    return m_pString[dwIndex];
}

wchar_t & UnicodeString::operator [] (unsigned long dwIndex) throw()
{
    return m_pString[dwIndex];
}

bool UnicodeString::IsCharAlphabetic(wchar_t ch) throw()
{
    return (ch >= L'a' && ch <= L'z') || (ch >= L'A' && ch <= L'Z');
}

bool UnicodeString::IsCharDigit(wchar_t ch) throw()
{
    return ch >= L'0' && ch <= L'9';
}

bool UnicodeString::IsCharAlphaDigit(wchar_t ch) throw()
{
    return (ch >= L'a' && ch <= L'z') || (ch >= L'A' && ch <= L'Z') || (ch >= L'0' && ch <= L'9');
}

bool UnicodeString::IsCharWhitespace(wchar_t ch,bool bIncludeCrLf) throw()
{
    bool bRet = ch == L' ' || ch == L'\t';
    if (bRet == false && bIncludeCrLf == true)
    {
        bRet = ch == L'\r' || ch == L'\n';
    }
    return bRet;
}

unsigned long UnicodeString::StringLength(const wchar_t * pString) throw()
{
    unsigned long dwLength(0);
    while (pString[dwLength] != 0)
    {
        ++dwLength;
    }
    return dwLength;
}

void UnicodeString::StringCopy(wchar_t * pstrBuffer,const wchar_t * pString,unsigned long dwCount) throw()
{
    for (unsigned long i = 0; i < dwCount; ++i)
    {
        pstrBuffer[i] = pString[i];
    }
}

}