///////////////////////////////////////////////
//// XTR.String.h
//// (C) 2006 James Bernard Schumacher III
///////////////////////////////////////////////

#ifndef __XTRSTRING_H__
#define __XTRSTRING_H__

#ifndef __cplusplus
#error XTRString requires C++ compilation. (use a .cpp suffix)
#endif // !defined __cplusplus

#include <vector>

#include <pshpack8.h>

namespace XTR
{

class AnsiString
{
public:
    friend class UnicodeString;
    // Construction/Destruction
    AnsiString() throw();
    explicit AnsiString(unsigned long dwBufferSize) throw();
    AnsiString(char ch) throw();
    AnsiString(const char * pString) throw();
    AnsiString(const AnsiString & str) throw();
    virtual ~AnsiString() throw();
    // Member Functions
    bool IsEmpty() const throw();
    void Clear() throw();
    unsigned long GetLength() const throw();
    unsigned long GetCapacity() const throw();
    bool SetCapacity(unsigned long dwSize,bool bKeepData) throw();
    const char * GetString() const throw();
    char * GetBuffer() throw();
    char * GetBuffer(unsigned long dwBufferSize) throw();
    bool SetAt(unsigned long dwIndex,char ch) throw();
    bool GetAt(unsigned long dwIndex,char & chRef) const throw();
    bool Append(char ch) throw();
    bool Append(char ch,unsigned long dwCount) throw();
    bool Append(const char * pString) throw();
    bool Append(const char * pString,unsigned long dwCharCount) throw();
    bool Append(const AnsiString & str) throw();
    bool Append(const AnsiString & str,unsigned long dwCharCount) throw();
    bool Append(const AnsiString * pStrings,unsigned long dwStringCount) throw();
    bool Prepend(char ch) throw();
    bool Prepend(char ch,unsigned long dwCount) throw();
    bool Prepend(const char * pString) throw();
    bool Prepend(const char * pString,unsigned long dwCharCount) throw();
    bool Prepend(const AnsiString & str) throw();
    bool Prepend(const AnsiString & str,unsigned long dwCharCount) throw();
    bool Prepend(const AnsiString * pStrings,unsigned long dwStringCount) throw();
    bool Insert(unsigned long dwIndex,char ch) throw();
    bool Insert(unsigned long dwIndex,char ch,unsigned long dwCount) throw();
    bool Insert(unsigned long dwIndex,const char * pString) throw();
    bool Insert(unsigned long dwIndex,const char * pString,unsigned long dwCharCount) throw();
    bool Insert(unsigned long dwIndex,const AnsiString & str) throw();
    bool Insert(unsigned long dwIndex,const AnsiString & str,unsigned long dwCharCount) throw();
    bool Find(char chFind,unsigned long & dwFoundIndex) const throw();
    bool Find(const char * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool Find(const AnsiString & strFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,char chFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,const char * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,const AnsiString & strFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(char chFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(const char * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(const AnsiString & strFind,unsigned long & dwFoundIndex) const throw();
    void FindAllInstancesOf(char chFind,std::vector<unsigned long> & arFoundIndices) const throw();
    void FindAllInstancesOf(const char * pstrFind,std::vector<unsigned long> & arFoundIndices) const throw();
    void FindAllInstancesOf(const AnsiString & strFind,std::vector<unsigned long> & arFoundIndices) const throw();
    bool FindOneOf(const char * pstrCharSet,unsigned long & dwFoundIndex) const throw();
    bool FindOneOf(const AnsiString & strCharSet,unsigned long & dwFoundIndex) const throw();
    unsigned long Replace(char chOld,char chNew) throw();
    unsigned long Replace(const char * pstrOld,const char * pstrNew) throw();
    unsigned long Replace(const AnsiString & strOld,const AnsiString & strNew) throw();
    unsigned long Replace(const char * pstrOld,const char * pstrNew,bool bWholeWord) throw();
    unsigned long Replace(const AnsiString & strOld,const AnsiString & strNew,bool bWholeWord) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwCount,char ch) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const char * pString) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const AnsiString & str) throw();
    unsigned long Remove(char chRemove) throw();
    unsigned long Remove(const char * pstrRemove) throw();
    unsigned long Remove(const AnsiString & strRemove) throw();
    unsigned long Remove(const char * pstrRemove,bool bWholeWordOnly) throw();
    unsigned long Remove(const AnsiString & strRemove,bool bWholeWordOnly) throw();
    bool RemoveAt(unsigned long dwIndex,unsigned long dwCount) throw();
    void MakeUppercase() throw();
    void MakeLowercase() throw();
    void SwitchCase() throw();
    void Reverse() throw();
    bool SubString(unsigned long dwStart,unsigned long dwCount,AnsiString & strBuffer) const throw();
    bool Extract(unsigned long dwStart,unsigned long dwCount,AnsiString & strBuffer) throw();
    bool SpanIncluding(const char * pstrCharSet,AnsiString & strBuffer) const throw();
    bool SpanIncluding(const AnsiString & strCharSet,AnsiString & strBuffer) const throw();
    bool SpanIncluding(unsigned long dwStart,const char * pstrCharSet,AnsiString & strBuffer) const throw();
    bool SpanIncluding(unsigned long dwStart,const AnsiString & strCharSet,AnsiString & strBuffer) const throw();
    bool SpanExcluding(const char * pstrCharSet,AnsiString & strBuffer) const throw();
    bool SpanExcluding(const AnsiString & strCharSet,AnsiString & strBuffer) const throw();
    bool SpanExcluding(unsigned long dwStart,const char * pstrCharSet,AnsiString & strBuffer) const throw();
    bool SpanExcluding(unsigned long dwStart,const AnsiString & strCharSet,AnsiString & strBuffer) const throw();
    void Scramble() throw();
    void Descramble() throw();
    void TrimLeft(char chTrim) throw();
    void TrimLeft(const char * pstrCharSet) throw();
    void TrimLeft(const AnsiString & strCharSet) throw();
    void TrimLeft() throw();
    void TrimRight(char chTrim) throw();
    void TrimRight(const char * pstrCharSet) throw();
    void TrimRight(const AnsiString & strCharSet) throw();
    void TrimRight() throw();
    UnicodeString ToUnicode() const throw();
    // Overloaded Operators
    const AnsiString & operator = (char ch) throw();
    const AnsiString & operator = (const char * pString) throw();
    const AnsiString & operator = (const AnsiString & str) throw();
    AnsiString & operator += (char ch) throw();
    AnsiString & operator += (const char * pString) throw();
    AnsiString & operator += (const AnsiString & str) throw();
    AnsiString operator + (char ch) const throw();
    AnsiString operator + (const char * pString) const throw();
    AnsiString operator + (const AnsiString & str) const throw();
    bool operator == (char ch) const throw();
    bool operator == (const char * pString) const throw();
    bool operator == (const AnsiString & str) const throw();
    bool operator != (char ch) const throw();
    bool operator != (const char * pString) const throw();
    bool operator != (const AnsiString & str) const throw();
    const char & operator [] (unsigned long dwIndex) const throw();
    char & operator [] (unsigned long dwIndex) throw();
    // Static Member Functions
    static bool IsCharAlphabetic(char ch) throw();
    static bool IsCharDigit(char ch) throw();
    static bool IsCharAlphaDigit(char ch) throw();
    static bool IsCharWhitespace(char ch,bool bIncludeCrLf) throw();
    static unsigned long StringLength(const char * pString) throw();
    static void StringCopy(char * pstrBuffer,const char * pString,unsigned long dwCount) throw();
private:
    // Member Variables
    char * m_pString;
    unsigned long m_dwBufferSize;
    // Private Implementation
    static unsigned long DetermineBufferSize(unsigned long dwCharCount) throw();
};

class UnicodeString
{
public:
    friend class AnsiString;
    // Construction/Destruction
    UnicodeString() throw();
    explicit UnicodeString(unsigned long dwBufferSize) throw();
    UnicodeString(wchar_t ch) throw();
    UnicodeString(const wchar_t * pString) throw();
    UnicodeString(const UnicodeString & str) throw();
    virtual ~UnicodeString() throw();
    // Member Functions
    bool IsEmpty() const throw();
    void Clear() throw();
    unsigned long GetLength() const throw();
    unsigned long GetCapacity() const throw();
    bool SetCapacity(unsigned long dwSize,bool bKeepData) throw();
    const wchar_t * GetString() const throw();
    wchar_t * GetBuffer() throw();
    wchar_t * GetBuffer(unsigned long dwBufferSize) throw();
    bool SetAt(unsigned long dwIndex,wchar_t ch) throw();
    bool GetAt(unsigned long dwIndex,wchar_t & chRef) const throw();
    bool Append(wchar_t ch) throw();
    bool Append(wchar_t ch,unsigned long dwCount) throw();
    bool Append(const wchar_t * pString) throw();
    bool Append(const wchar_t * pString,unsigned long dwCharCount) throw();
    bool Append(const UnicodeString & str) throw();
    bool Append(const UnicodeString & str,unsigned long dwCharCount) throw();
    bool Append(const UnicodeString * pStrings,unsigned long dwStringCount) throw();
    bool Prepend(wchar_t ch) throw();
    bool Prepend(wchar_t ch,unsigned long dwCount) throw();
    bool Prepend(const wchar_t * pString) throw();
    bool Prepend(const wchar_t * pString,unsigned long dwCharCount) throw();
    bool Prepend(const UnicodeString & str) throw();
    bool Prepend(const UnicodeString & str,unsigned long dwCharCount) throw();
    bool Prepend(const UnicodeString * pStrings,unsigned long dwStringCount) throw();
    bool Insert(unsigned long dwIndex,wchar_t ch) throw();
    bool Insert(unsigned long dwIndex,wchar_t ch,unsigned long dwCount) throw();
    bool Insert(unsigned long dwIndex,const wchar_t * pString) throw();
    bool Insert(unsigned long dwIndex,const wchar_t * pString,unsigned long dwCharCount) throw();
    bool Insert(unsigned long dwIndex,const UnicodeString & str) throw();
    bool Insert(unsigned long dwIndex,const UnicodeString & str,unsigned long dwCharCount) throw();
    bool Find(wchar_t chFind,unsigned long & dwFoundIndex) const throw();
    bool Find(const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool Find(const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,wchar_t chFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool FindFrom(unsigned long dwStart,const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(wchar_t chFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(const wchar_t * pstrFind,unsigned long & dwFoundIndex) const throw();
    bool FindLastInstanceOf(const UnicodeString & strFind,unsigned long & dwFoundIndex) const throw();
    void FindAllInstancesOf(wchar_t chFind,std::vector<unsigned long> & arFoundIndices) const throw();
    void FindAllInstancesOf(const wchar_t * pstrFind,std::vector<unsigned long> & arFoundIndices) const throw();
    void FindAllInstancesOf(const UnicodeString & strFind,std::vector<unsigned long> & arFoundIndices) const throw();
    bool FindOneOf(const wchar_t * pstrCharSet,unsigned long & dwFoundIndex) const throw();
    bool FindOneOf(const UnicodeString & strCharSet,unsigned long & dwFoundIndex) const throw();
    unsigned long Replace(wchar_t chOld,wchar_t chNew) throw();
    unsigned long Replace(const wchar_t * pstrOld,const wchar_t * pstrNew) throw();
    unsigned long Replace(const UnicodeString & strOld,const UnicodeString & strNew) throw();
    unsigned long Replace(const wchar_t * pstrOld,const wchar_t * pstrNew,bool bWholeWordOnly) throw();
    unsigned long Replace(const UnicodeString & strOld,const UnicodeString & strNew,bool bWholeWordOnly) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwOldCount,wchar_t ch) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwOldCount,const wchar_t * pString) throw();
    bool ReplaceAt(unsigned long dwIndex,unsigned long dwOldCount,const UnicodeString & str) throw();
    unsigned long Remove(wchar_t chRemove) throw();
    unsigned long Remove(const wchar_t * pstrRemove) throw();
    unsigned long Remove(const UnicodeString & strRemove) throw();
    unsigned long Remove(const wchar_t * pstrRemove,bool bWholeWordOnly) throw();
    unsigned long Remove(const UnicodeString & strRemove,bool bWholeWordOnly) throw();
    bool RemoveAt(unsigned long dwStart,unsigned long dwCount) throw();
    void MakeUppercase() throw();
    void MakeLowercase() throw();
    void SwitchCase() throw();
    void Reverse() throw();
    bool SubString(unsigned long dwStart,unsigned long dwCount,UnicodeString & strBuffer) const throw();
    bool Extract(unsigned long dwStart,unsigned long dwCount,UnicodeString & strBuffer) throw();
    bool SpanIncluding(const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw();
    bool SpanIncluding(const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw();
    bool SpanIncluding(unsigned long dwStart,const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw();
    bool SpanIncluding(unsigned long dwStart,const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw();
    bool SpanExcluding(const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw();
    bool SpanExcluding(const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw();
    bool SpanExcluding(unsigned long dwStart,const wchar_t * pstrCharSet,UnicodeString & strBuffer) const throw();
    bool SpanExcluding(unsigned long dwStart,const UnicodeString & strCharSet,UnicodeString & strBuffer) const throw();
    void Scramble() throw();
    void Descramble() throw();
    void TrimLeft(wchar_t chTrim) throw();
    void TrimLeft(const wchar_t * pstrCharSet) throw();
    void TrimLeft(const UnicodeString & strCharSet) throw();
    void TrimLeft() throw();
    void TrimRight(wchar_t chTrim) throw();
    void TrimRight(const wchar_t * pstrCharSet) throw();
    void TrimRight(const UnicodeString & strCharSet) throw();
    void TrimRight() throw();
    AnsiString ToAnsi() const throw();
    // Overloaded Operators
    const UnicodeString & operator = (wchar_t ch) throw();
    const UnicodeString & operator = (const wchar_t * pString) throw();
    const UnicodeString & operator = (const UnicodeString & str) throw();
    UnicodeString & operator += (wchar_t ch) throw();
    UnicodeString & operator += (const wchar_t * pString) throw();
    UnicodeString & operator += (const UnicodeString & str) throw();
    UnicodeString operator + (wchar_t ch) const throw();
    UnicodeString operator + (const wchar_t * pString) const throw();
    UnicodeString operator + (const UnicodeString & str) const throw();
    bool operator == (wchar_t ch) const throw();
    bool operator == (const wchar_t * pString) const throw();
    bool operator == (const UnicodeString & str) const throw();
    bool operator != (wchar_t ch) const throw();
    bool operator != (const wchar_t * pString) const throw();
    bool operator != (const UnicodeString & str) const throw();
    const wchar_t & operator [] (unsigned long dwIndex) const throw();
    wchar_t & operator [] (unsigned long dwIndex) throw();
    // Static Member Functions
    static bool IsCharAlphabetic(wchar_t ch) throw();
    static bool IsCharDigit(wchar_t ch) throw();
    static bool IsCharAlphaDigit(wchar_t ch) throw();
    static bool IsCharWhitespace(wchar_t ch,bool bIncludeCrLf) throw();
    static unsigned long StringLength(const wchar_t * pString) throw();
    static void StringCopy(wchar_t * pstrBuffer,const wchar_t * pString,unsigned long dwCount) throw();
private:
    // Member Variables
    wchar_t * m_pString;
    unsigned long m_dwBufferSize;
};

}

#include <poppack.h>

#endif // !defined __XTRSTRING_H__
